import logging
from datetime import datetime
from .hubmember import HubMember
from .const import CURRENTPEAKSENSOR

_LOGGER = logging.getLogger(__name__)


class CurrentPeak(HubMember):
    def __init__(self, data_type: type, initval, startpeaks:dict):
        self._startpeak = self._set_start_peak(startpeaks)
        self._value = initval
        super().__init__(data_type, initval)

    def _set_start_peak(self, peaks:dict) -> float:
        peak = peaks.get(datetime.now().month)
        if peak is None:
            peak = peaks.get(str(datetime.now().month))
            if peak is None:
                raise ValueError
        return peak

    @HubMember.value.getter
    def value(self): # pylint:disable=invalid-overridden-method
        return max(self._value, float(self._startpeak)) if self._value is not None else float(self._startpeak)




from dataclasses import dataclass, field
from typing import List
from .enums.datepart_datetype import DatePartDateType
from .enums.datepart_modeltype import DatePartModelType
from .enums.dividents import Dividents

@dataclass
class DatePartModel:
    type: DatePartModelType = field(default_factory=lambda : DatePartModelType.Unset)
    dttype:  DatePartDateType = field(default_factory=lambda : DatePartDateType.Unset)
    values: List[int] = field(default_factory=lambda : [])

@dataclass
class Group:
    divident: Dividents = field(default_factory=lambda: Dividents.UNSET)
    dateparts: List[DatePartModel] = field(default_factory=lambda : [DatePartModel()])

@dataclass
class QueryServiceModel:
    groups: List[Group] = field(default_factory=lambda : [Group()])



from dataclasses import dataclass, field
from abc import ABC, abstractmethod
from ..models.hub.hubmember import HubMember
from ..models.hub.currentpeak import CurrentPeak
from ..models.hub.carpowersensor import CarPowerSensor
from ..models.hub.chargerobject import ChargerObject
from ..models.hub.chargerswitch import ChargerSwitch
from ..services.locale.Locale import LocaleData
from ..models.hub.power import Power
from .hub_options import HubOptions
from ..util import nametoid
from typing import Any
from .const import HOURLY
from ..models.hub.const import CONSUMPTION_TOTAL_NAME, AVERAGECONSUMPTION, AVERAGECONSUMPTION_24H, CHARGERDONE, CHARGERENABLED


@dataclass
class IHubSensors:
    charger_enabled: HubMember = field(init=False)
    charger_done: HubMember = field(init=False)
    current_peak: CurrentPeak = field(init=False)
    totalhourlyenergy: HubMember = field(init=False)
    carpowersensor: CarPowerSensor = field(init=False)
    locale: LocaleData = field(init=False)
    chargerobject: ChargerObject = field(init=False)
    chargerobject_switch: ChargerSwitch = field(init=False)
    state_machine: Any = field(init=False)
    powersensormovingaverage: HubMember = field(init=False)
    powersensormovingaverage24: HubMember = field(init=False)
    power: Power = field(init=False)

    @abstractmethod
    def setup(self):
        pass

    def setup_base(
            self,
            options: HubOptions,
            state_machine,
            domain: str,
            chargerobject
    ):
        self.chargertype = chargerobject
        self.state_machine = state_machine
        resultdict = {}

        self.charger_enabled = HubMember(
            data_type=bool,
            listenerentity=f"switch.{domain}_{nametoid(CHARGERENABLED)}",
            initval=False
        )
        self.charger_done = HubMember(
            data_type=bool,
            listenerentity=f"binary_sensor.{domain}_{nametoid(CHARGERDONE)}",
            initval=False
        )
        self.locale = LocaleData(
            options.locale,
            domain
        )
        self.current_peak = CurrentPeak(
            data_type=float,
            initval=0,
            startpeaks=options.startpeaks,
        )
        if len(self.chargertype.charger.entities.chargerentity):
            self.chargerobject = ChargerObject(
                data_type=self.chargertype.charger.native_chargerstates,
                listenerentity=self.chargertype.charger.entities.chargerentity
            )
            resultdict[self.chargerobject.entity] = self.chargerobject.is_initialized

            self.carpowersensor = CarPowerSensor(
                data_type=int,
                listenerentity=self.chargertype.charger.entities.powermeter,
                powermeter_factor=self.chargertype.charger.options.powermeter_factor,
                hubdata=self,
                init_override=True
            )
            self.chargerobject_switch = ChargerSwitch(
                hass=state_machine,
                data_type=bool,
                listenerentity=self.chargertype.charger.entities.powerswitch,
                initval=False,
                currentname=self.chargertype.charger.entities.ampmeter,
                ampmeter_is_attribute=self.chargertype.charger.options.ampmeter_is_attribute,
                hubdata=self,
                init_override=True
            )

        else:
            self.chargerobject = ChargerObject(
                data_type=self.chargertype.charger.native_chargerstates,
                listenerentity="no entity",
                init_override=True
            )
            resultdict[self.chargerobject.entity] = True
            
            self.carpowersensor = CarPowerSensor(
                data_type=int,
                listenerentity=self.chargertype.charger.entities.powermeter,
                powermeter_factor=self.chargertype.charger.options.powermeter_factor,
                hubdata=self
            )
            self.chargerobject_switch = ChargerSwitch(
                hass=state_machine,
                data_type=bool,
                listenerentity=self.chargertype.charger.entities.powerswitch,
                initval=False,
                currentname=self.chargertype.charger.entities.ampmeter,
                ampmeter_is_attribute=self.chargertype.charger.options.ampmeter_is_attribute,
                hubdata=self
            )
        self.totalhourlyenergy = HubMember(
            data_type=float,
            listenerentity=f"sensor.{domain}_{nametoid(CONSUMPTION_TOTAL_NAME)}_{HOURLY}",
            initval=0
        )

    def init_hub_values(self):
        """Initialize values from Home Assistant on the set objects"""
        if self.chargerobject is not None:
            self.chargerobject.value = self.state_machine.states.get(self.chargerobject.entity).state if self.state_machine.states.get(self.chargerobject.entity) is not None else 0
        self.chargerobject_switch.value = self.state_machine.states.get(self.chargerobject_switch.entity).state if self.state_machine.states.get(self.chargerobject_switch.entity) is not None else ""
        self.chargerobject_switch.updatecurrent()
        self.carpowersensor.value = self.state_machine.states.get(self.carpowersensor.entity).state if isinstance(self.state_machine.states.get(self.carpowersensor.entity),(float,int)) else 0
        self.totalhourlyenergy.value = self.state_machine.states.get(self.totalhourlyenergy.entity) if isinstance(self.state_machine.states.get(self.totalhourlyenergy.entity),(float,int)) else 0


@dataclass
class HubSensorsLite(IHubSensors):
   def setup(
        self,
        state_machine,
        options: HubOptions,
        domain: str, 
        chargerobject: Any):

        super().setup_base(state_machine=state_machine, options=options, domain=domain, chargerobject=chargerobject)


@dataclass
class HubSensors(IHubSensors):
    def setup(
        self,
        state_machine,
        options: HubOptions,
        domain: str, 
        chargerobject: Any):

        super().setup_base(state_machine=state_machine, options=options, domain=domain, chargerobject=chargerobject)

        self.powersensormovingaverage = HubMember(
            data_type=int,
            listenerentity=f"sensor.{domain}_{nametoid(AVERAGECONSUMPTION)}",
            initval=0
        )
        self.powersensormovingaverage24 = HubMember(
            data_type=int,
            listenerentity=f"sensor.{domain}_{nametoid(AVERAGECONSUMPTION_24H)}",
            initval=0
        )

        self.power = Power(
            configsensor=options.powersensor,
            powersensor_includes_car=options.powersensor_includes_car
        )

class HubSensorsFactory:
    @staticmethod
    def create(options: HubOptions) -> IHubSensors:
        if options.peaqev_lite:
            return HubSensorsLite()
        return HubSensors()






from abc import abstractmethod
from dataclasses import dataclass, field
from ...models.chargertype.servicecalls_dto import ServiceCallsDTO
from ...models.chargertype.servicecalls_options import ServiceCallsOptions
from ...models.chargecontroller_states import ChargeControllerStates
from ...models.chargertype.charger_options import ChargerOptions
from ...models.chargertype.calltype import CallType
from ...models.chargertype.charger_entities_model import ChargerEntitiesModel
from .servicecalls import ServiceCalls

CHARGERSTATES_BASE = {
    ChargeControllerStates.Idle: [],
    ChargeControllerStates.Connected: [],
    ChargeControllerStates.Charging: [],
    ChargeControllerStates.Done: []
}

@dataclass
class ChargerBase:
    domainname:str = ""
    _max_amps = None
    native_chargerstates:list = field(default_factory=lambda: [])
    servicecalls:ServiceCalls = None
    chargerstates = CHARGERSTATES_BASE
    entities = ChargerEntitiesModel
    options = ChargerOptions

    def _set_servicecalls(
            self,
            domain: str,
            model: ServiceCallsDTO,
            options: ServiceCallsOptions,
    ) -> None:
        self.servicecalls = ServiceCalls(domain, model, options)

    @property
    def max_amps(self) -> int:
        if self._max_amps is None:
            return 16
        return self._max_amps

    @max_amps.setter
    def max_amps(self, val) -> None:
        self._max_amps = val

    @abstractmethod
    def get_allowed_amps(self) -> int:
        pass

    @abstractmethod
    def validatecharger(self) -> bool:
        pass

    @abstractmethod
    def get_entities(self):
        pass

    @abstractmethod
    def set_sensors(self):
        pass

    @property
    @abstractmethod
    def domain_name(self) -> str:
        """declare the domain name as stated in HA"""
        pass

    @property
    @abstractmethod
    def entity_endings(self) -> list:
        """declare a list of strings with sensor-endings to help peaqev find the correct sensor-schema."""
        pass

    @property
    @abstractmethod
    def native_chargerstates(self) -> list:
        """declare a list of the native-charger states available for the type."""
        pass

    @property
    @abstractmethod
    def call_on(self) -> CallType:
        pass

    @property
    @abstractmethod
    def call_off(self) -> CallType:
        pass

    @property
    @abstractmethod
    def call_resume(self) -> CallType:
        pass

    @property
    @abstractmethod
    def call_pause(self) -> CallType:
        pass

    @property
    @abstractmethod
    def call_update_current(self) -> CallType:
        pass

    @property
    @abstractmethod
    def servicecalls_options(self) -> ServiceCallsOptions:
        pass
    


from ...models.chargecontroller_states import ChargeControllerStates
from .chargecontrollerbase import ChargeControllerBase


class ChargeControllerLite(ChargeControllerBase):
    def __init__(self, hub):
        super().__init__(hub)

    def _get_status_charging(self) -> ChargeControllerStates:
        if self._hub.totalhourlyenergy.value >= self._hub.current_peak_dynamic and self._hub.locale.data.free_charge(self._hub.locale.data) is False:
            ret = ChargeControllerStates.Stop
        else:
            ret = ChargeControllerStates.Start
        return ret

    def _get_status_connected(self, charger_state = None) -> ChargeControllerStates:
        if charger_state is not None and self._hub.sensors.carpowersensor.value < 1 and self._is_done(charger_state):
            ret = ChargeControllerStates.Done
        else:
            if (self._hub.totalhourlyenergy.value < self._hub.current_peak_dynamic) or self._hub.locale.data.free_charge(self._hub.locale.data) is True:
                ret = ChargeControllerStates.Start
            else:
                ret = ChargeControllerStates.Stop
        return ret
